-- Example: draw rectangles with rounded corners

local rrect = require("rect")

local fps = 60
local urate = 1 / fps
local sw = 800
local black = {0, 0, 0}
local white = {255, 255, 255}
local red = {255, 0, 0}
local green = {0, 255, 0}
local blue = {0, 0, 255}
local yellow = {255, 0, 255}
local magenta = {255, 255, 0}
local cyan = {0, 255, 255}
local light_red = {255, 128, 128, 128}
local light_green = {128, 255, 128, 128}

function save_screenshot(filename)
    print(("Saving screenshot to '%s'..."):format(filename))
    local screenshot = love.graphics.newScreenshot()
    screenshot:encode(filename)
end

function draw_rects(rects, aa, step)
    for _, rect in pairs(rects) do
        rect.canvas = rrect.draw_rect("line", rect.w, rect.h, rect.radius,
            rect.color, rect.bgcolor, rect.linewidth, aa or 0, step or .1)
    end
end

function love.load()
    background = love.graphics.newImage("background.png")
    bw, bh = background:getDimensions()
    sh = sw * (bh / bw)

    antialias = 4
    love.filesystem.setIdentity("roundedrect")
    love.window.setMode(sw, sh, {fsaa = antialias, vsync = true})
    font = love.graphics.newFont(18)
    lineheight = font:getHeight()

    rects = {
        {x = 30, y = 80, w = 400, h = 200, radius = 10, linewidth = 3,
         color = red, bgcolor = light_red},
        {x = 550, y = 350, w = 200, h = 200, radius = 10, linewidth = 3,
         color = cyan, bgcolor = light_green}
    }
    draw_rects(rects, antialias)

    love.graphics.setFont(font)
    love.graphics.setColor(unpack(white))
    love.graphics.setBackgroundColor(0, 0, 0)
    love.resize(sw, sh)
end

function love.resize(w, h)
    scale = w > h and w / bw or h / bh
end

function love.keypressed(key)
    if key == "f" then
        love.window.setFullscreen(not love.window.getFullscreen(), "desktop")
    elseif key == "s" then
        save_screenshot("screenshot-" .. os.date("%Y%m%d-%H%M%S") .. ".png")
    elseif key == "escape" or key == "q" then
        love.event.quit()
    else
        number = key:match("[0124]")
        if number then
            antialias = tonumber(number)
            -- doesn't work correctly on my graphics chip
            draw_rects(rects, antialias)
        end
    end
end

function love.update(dt)
    if dt < (urate) then
        love.timer.sleep(urate - dt - 0.001)
    end
end

function love.draw()
    local ofc = {love.graphics.getColor()}
    local obc = {love.graphics.getBackgroundColor()}

    -- draw scaled background
    love.graphics.scale(scale, scale)
    love.graphics.draw(background)
    love.graphics.origin()

    -- draw rectangles
    love.graphics.setBlendMode('premultiplied')
    for _, rect in pairs(rects) do
        -- offset rectangle position by half of linewidth
        -- to account for extra width / height added by border
        local hlw = rect.linewidth / 2
        love.graphics.draw(rect.canvas, rect.x - hlw, rect.y - hlw)
    end
    love.graphics.setBackgroundColor(unpack(obc))
    love.graphics.setColor(unpack(ofc))
    love.graphics.setBlendMode('alpha')

    -- print some info
    love.graphics.print(("FPS: %02i"):format(love.timer.getFPS()), 40, 90)
    love.graphics.print(("Screen: %ix%i"):format(love.window.getDimensions()),
                        40, 90 + lineheight)
    love.graphics.print(("Anti-alias: %i samples"):format(antialias),
                        40, 90 + lineheight * 2)
    love.graphics.print("Press 'f' to toggle fullscreen.",
                        40, 90 + lineheight * 4)
    love.graphics.print("'q/ESC' to quit and 's' to save a screenshot.",
                        40, 90 + lineheight * 5)
end
